'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _loreReactForms = require('lore-react-forms');

var _string = require('./fields/string');

var _string2 = _interopRequireDefault(_string);

var _dynamicString = require('./fields/dynamicString');

var _dynamicString2 = _interopRequireDefault(_dynamicString);

var _select = require('./fields/select');

var _select2 = _interopRequireDefault(_select);

var _autocomplete = require('./fields/autocomplete');

var _autocomplete2 = _interopRequireDefault(_autocomplete);

var _cancel = require('./actions/cancel');

var _cancel2 = _interopRequireDefault(_cancel);

var _submit = require('./actions/submit');

var _submit2 = _interopRequireDefault(_submit);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {

  dependencies: ['models'],

  defaults: {
    forms: {
      templates: {
        default: _loreReactForms.Template
      },

      typeFieldMap: {
        string: _string2.default,
        dynamicString: _dynamicString2.default,
        select: _select2.default,
        autocomplete: _autocomplete2.default
      },

      typeActionMap: {
        cancel: _cancel2.default,
        submit: _submit2.default
      }
    }
  },

  load: function load(lore) {
    var schemas = lore.loader.loadModels();

    lore.forms = {};

    _lodash2.default.mapKeys(schemas, function (schema, modelName) {
      lore.forms[modelName] = lore.forms[modelName] || {};

      lore.forms[modelName].create = function (props) {
        var Template = lore.config.forms.templates[props.template || 'default'];
        var templateProps = _lodash2.default.merge({}, schema.forms, props);
        return _react2.default.createElement(Template, templateProps);
      };

      lore.forms[modelName].update = function (model, props) {
        var Template = lore.config.forms.templates[props.template || 'default'];

        // get an object with all fields that will passed to the form
        var tempProps = _lodash2.default.merge({}, schema.forms, props);

        // populate the data attribute of each field with the model's data
        var modelFieldsData = {
          fields: _lodash2.default.mapValues(tempProps.fields, function (value, key) {
            return {
              data: model.data[key]
            };
          })
        };

        // create the final props for the template, overwriting the model data with provided data
        var templateProps = _lodash2.default.merge({}, schema.forms, modelFieldsData, props);
        return _react2.default.createElement(Template, templateProps);
      };
    });
  }

}; /* eslint no-param-reassign: "off" */
/* eslint react/prop-types: "off" */

module.exports = exports['default'];